﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("LaplacianKernel", "A kernel function that uses an exponential function.\nIt is equvalent to the Gaussian kernel but less susceptible to improper values of beta")]
  public class LaplacianKernel : KernelBase {
    [StorableConstructor]
    protected LaplacianKernel(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }
    protected LaplacianKernel(LaplacianKernel original, Cloner cloner) : base(original, cloner) { }
    public LaplacianKernel() {
      Parameters.Add(new FixedValueParameter<DoubleValue>(BetaParameterName, "The beta in the kernel function exp(-||x-c||/beta)", new DoubleValue(2)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LaplacianKernel(this, cloner);
    }

    protected override double Get(double norm) {
      return Math.Abs(Beta) < double.Epsilon ? 0 : Math.Exp(-norm / Beta);
    }

    protected override double GetGradient(double norm) {
      if (Math.Abs(Beta) < double.Epsilon) return double.NaN;
      return norm * Math.Exp(-norm / Beta) / (Beta * Beta);
    }
  }
}
