﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("PolysplineKernel", "A kernel function that uses the multiquadratic function")]
  public class PolysplineKernel : KernelBase {

    #region HLConstructors & Boilerplate
    [StorableConstructor]
    protected PolysplineKernel(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }
    protected PolysplineKernel(PolysplineKernel original, Cloner cloner)
                : base(original, cloner) { }
    public PolysplineKernel() {
      Parameters.Add(new FixedValueParameter<DoubleValue>(BetaParameterName, "The Beta in the kernelfunction ||x-c||^Beta", new DoubleValue(1.5)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PolysplineKernel(this, cloner);
    }
    #endregion

    protected override double Get(double norm) {
      return Math.Pow(norm, Beta);
    }

    protected override double GetGradient(double norm) {
      return Math.Pow(norm, Beta) * Math.Log(Beta);
    }
  }
}
